# WindowAdisTS.py -- Pamhyr
# Copyright (C) 2023-2024  INRAE
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

# -*- coding: utf-8 -*-

import os
import csv
import logging

from datetime import datetime
from tools import trace, timer, logger_exception

from View.Tools.PamhyrWindow import PamhyrWindow

from PyQt5.QtGui import (
    QKeySequence, QIcon, QPixmap,
)

from PyQt5.QtCore import (
    Qt, QVariant, QAbstractTableModel,
    QCoreApplication, QModelIndex, pyqtSlot,
    QItemSelectionModel, QTimer,
)

from PyQt5.QtWidgets import (
    QDialogButtonBox, QPushButton, QLineEdit,
    QFileDialog, QTableView, QAbstractItemView,
    QUndoStack, QShortcut, QAction, QItemDelegate,
    QComboBox, QVBoxLayout, QHeaderView, QTabWidget,
    QSlider, QLabel, QWidget, QGridLayout,
)

from View.Tools.Plot.PamhyrCanvas import MplCanvas
from View.Tools.Plot.PamhyrToolbar import PamhyrPlotToolbar

from View.Results.PlotCAdisTS import PlotC
from View.Results.PlotMAdisTS import PlotM
from View.Results.PlotTotSedCAdisTS import PlotTotSedC
from View.Results.PlotTotSedEGAdisTS import PlotTotSedEG
from View.Results.PlotTotSedEMAdisTS import PlotTotSedEM
from View.Results.PlotTotSedEDAdisTS import PlotTotSedED

from View.Results.CustomPlot.Plot import CustomPlot
from View.Results.CustomPlot.CustomPlotValuesSelectionDialog import (
    CustomPlotValuesSelectionDialog,
)

from View.Results.TableAdisTS import TableModel
from View.Results.translate import ResultsTranslate

_translate = QCoreApplication.translate

logger = logging.getLogger()


class ResultsWindowAdisTS(PamhyrWindow):
    _pamhyr_ui = "ResultsAdisTS"
    _pamhyr_name = "Results"

    def _path_file(self, filename):
        return os.path.abspath(
            os.path.join(
                os.path.dirname(__file__),
                "..", "ui", "ressources", filename
            )
        )

    def __init__(self, study=None, config=None,
                 solver=None, results=None,
                 parent=None):
        self._solver = solver
        self._results = results

        trad = ResultsTranslate()
        name = (
            trad[self._pamhyr_name] + " - "
            + study.name + " - "
            + self._solver.name
        )

        super(ResultsWindowAdisTS, self).__init__(
            title=name,
            study=study,
            config=config,
            trad=trad,
            parent=parent
        )

        self._hash_data.append(self._solver)
        self._hash_data.append(self._results)

        self._additional_plot = {}

        try:
            #print("timestamps results: ", self._results)
            self._timestamps = sorted(list(self._results.get("timestamps")))

            print("setup table adists results")

            self.setup_table()
            self.setup_plots()
            self.setup_slider()
            print("///setup slider correct")
            self.setup_statusbar()
            print("///setup status bar correct")
            self.setup_connections()
            print("///setup connections correct")
        except Exception as e:
            logger_exception(e)
            return

    def setup_table(self):
        print("setup table adists results")
        print("results study name: ", self._results.study.name)
        print("results study river: ", self._results.study.river)
        print("*********files names resultats from Results Window: ", self._results.pollutants_list)
        self._table = {}
        for t in ["reach", "profile", "pollutants", "phys_var"]:#, "raw_data"]:
            print("t: ", t)
            table = self.find(QTableView, f"tableView_{t}")
            self._table[t] = TableModel(
                table_view=table,
                table_headers=self._trad.get_dict(f"table_headers_{t}"),
                data=self._results,
                undo=self._undo_stack,
                opt_data=t
            )

    def setup_slider(self):
        self._timer = QTimer(self)

    def setup_plots(self):
        self.canvas_4 = MplCanvas(width=5, height=4, dpi=100)
        self.canvas_4.setObjectName("canvas_4")
        self.toolbar_4 = PamhyrPlotToolbar(
            self.canvas_4, self, items=[
                "home", "move", "zoom", "save",
                "iso", "back/forward"
            ]
        )
        self.plot_layout_4 = self.find(
            QVBoxLayout, "verticalLayout_concentration")
        self.plot_layout_4.addWidget(self.toolbar_4)
        self.plot_layout_4.addWidget(self.canvas_4)

        self.plot_c = PlotC(
            canvas=self.canvas_4,
            results=self._results,
            reach_id=0,
            profile_id=0,
            pol_id=0,
            trad=self._trad,
            toolbar=self.toolbar_4
        )
        self.plot_c.draw()

        print("///plot c adists correct")

        self.canvas_3 = MplCanvas(width=5, height=4, dpi=100)
        self.canvas_3.setObjectName("canvas_3")
        self.toolbar_3 = PamhyrPlotToolbar(
            self.canvas_3, self, items=[
                "home", "move", "zoom", "save",
                "iso", "back/forward"
            ]
        )
        self.plot_layout_3 = self.find(
            QVBoxLayout, "verticalLayout_mass")
        self.plot_layout_3.addWidget(self.toolbar_3)
        self.plot_layout_3.addWidget(self.canvas_3)

        self.plot_m = PlotM(
            canvas=self.canvas_3,
            results=self._results,
            reach_id=0,
            profile_id=0,
            pol_id=0,
            trad=self._trad,
            toolbar=self.toolbar_3
        )
        self.plot_m.draw()

        print("///plot m adists correct")

        self.canvas_2 = MplCanvas(width=5, height=4, dpi=100)
        self.canvas_2.setObjectName("canvas_2")
        self.toolbar_2 = PamhyrPlotToolbar(
            self.canvas_2, self, items=[
                "home", "move", "zoom", "save",
                "iso", "back/forward"
            ]
        )
        self.plot_layout_2 = self.find(
            QVBoxLayout, "verticalLayout_tot_c")
        #self.plot_layout_2.addWidget(self.toolbar_2)
        self.plot_layout_2.addWidget(self.canvas_2)

        self.plot_tot_c = PlotTotSedC(
            canvas=self.canvas_2,
            results=self._results,
            reach_id=0,
            profile_id=0,
            pol_id=self._results.pollutants_list.index("total_sediment"),
            trad=self._trad,
            #toolbar=self.toolbar_2
        )

        self.plot_tot_c.draw()

        self.canvas_1 = MplCanvas(width=5, height=4, dpi=100)
        self.canvas_1.setObjectName("canvas_1")
        self.toolbar_1 = PamhyrPlotToolbar(
            self.canvas_1, self, items=[
                "home", "move", "zoom", "save",
                "iso", "back/forward"
            ]
        )
        self.plot_layout_1 = self.find(
            QVBoxLayout, "verticalLayout_tot_left_2")
        # self.plot_layout_1.addWidget(self.toolbar_1)
        self.plot_layout_1.addWidget(self.canvas_1)

        self.plot_tot_eg = PlotTotSedEG(
            canvas=self.canvas_1,
            results=self._results,
            reach_id=0,
            profile_id=0,
            pol_id=self._results.pollutants_list.index("total_sediment"),
            trad=self._trad,
            #toolbar=self.toolbar_1
        )

        self.plot_tot_eg.draw()

        self.canvas_0 = MplCanvas(width=5, height=4, dpi=100)
        self.canvas_0.setObjectName("canvas_0")
        self.toolbar_0 = PamhyrPlotToolbar(
            self.canvas_0, self, items=[
                "home", "move", "zoom", "save",
                "iso", "back/forward"
            ]
        )
        self.plot_layout_0 = self.find(
            QVBoxLayout, "verticalLayout_tot_minor")
        # self.plot_layout_0.addWidget(self.toolbar_0)
        self.plot_layout_0.addWidget(self.canvas_0)

        self.plot_tot_em = PlotTotSedEM(
            canvas=self.canvas_0,
            results=self._results,
            reach_id=0,
            profile_id=0,
            pol_id=self._results.pollutants_list.index("total_sediment"),
            trad=self._trad,
            # toolbar=self.toolbar_0
        )

        self.plot_tot_em.draw()

        self.canvas_5 = MplCanvas(width=5, height=4, dpi=100)
        self.canvas_5.setObjectName("canvas_5")
        self.toolbar_5 = PamhyrPlotToolbar(
            self.canvas_5, self, items=[
                "home", "move", "zoom", "save",
                "iso", "back/forward"
            ]
        )
        self.plot_layout_5 = self.find(
            QVBoxLayout, "verticalLayout_tot_right")
        # self.plot_layout_5.addWidget(self.toolbar_5)
        self.plot_layout_5.addWidget(self.canvas_5)

        self.plot_tot_ed = PlotTotSedED(
            canvas=self.canvas_5,
            results=self._results,
            reach_id=0,
            profile_id=0,
            pol_id=self._results.pollutants_list.index("total_sediment"),
            trad=self._trad,
            # toolbar=self.toolbar_0
        )

        self.plot_tot_ed.draw()

    def closeEvent(self, event):
        try:
            self._timer.stop()
        except Exception as e:
            logger_exception(e)

        super(ResultsWindowAdisTS, self).closeEvent(event)

    def _compute_status_label(self):
        # Reach
        table = self.find(QTableView, f"tableView_reach")
        indexes = table.selectedIndexes()
        if len(indexes) == 0:
            reach = self._study.river.edges()[0]
        else:
            reach = self._study.river.edges()[indexes[0].row()]

        # Profile
        table = self.find(QTableView, f"tableView_profile")
        indexes = table.selectedIndexes()
        if len(indexes) == 0:
            profile = reach.reach.profile(0)
        else:
            profile = reach.reach.profile(indexes[0].row())

        pname = profile.name if profile.name != "" else profile.kp

        # Pollutant
        table = self.find(QTableView, f"tableView_pollutants")
        indexes = table.selectedIndexes()
        if len(indexes) == 0:
            pollutant = self._results.pollutants_list[0]
        else:
            pollutant = self._results.pollutants_list[indexes[0].row()]

        return (f"Reach: {reach.name} | " +
                f"Profile: {pname} | " +
                f"Pollutant: {pollutant}")

    def setup_statusbar(self):
        txt = self._compute_status_label()
        self._status_label = QLabel(txt)
        self.statusbar.addPermanentWidget(self._status_label)

    def update_statusbar(self):
        txt = self._compute_status_label()
        self._status_label.setText(txt)

    def setup_connections(self):
        # Action
        actions = {
            "action_reload": self._reload,
            ###"action_add": self._add_custom_plot,
            "action_export": self.export,
        }

        for action in actions:
            self.find(QAction, action).triggered.connect(
                actions[action]
            )

        # Table and Plot
        fun = {
            "reach": self._set_current_reach,
            "profile": self._set_current_profile,
            "pollutants": self._set_current_pol,
            "raw_data": self._set_current_profile_raw_data,
        }

        for t in ["reach", "profile", "pollutants"]:###, "raw_data"]:
            table = self.find(QTableView, f"tableView_{t}")

            table.selectionModel()\
                 .selectionChanged\
                 .connect(fun[t])

            self._table[t].dataChanged.connect(fun[t])

        self._timer.timeout.connect(self._update_slider)

    def update_table_selection_reach(self, ind):
        table = self.find(QTableView, f"tableView_reach")
        selectionModel = table.selectionModel()
        index = table.model().index(ind, 0)

        selectionModel.select(
            index,
            QItemSelectionModel.Rows |
            QItemSelectionModel.ClearAndSelect |
            QItemSelectionModel.Select
        )
        table.scrollTo(index)

        self._table["profile"].update(ind)
        ###self._table["raw_data"].update(ind)

    def update_table_selection_profile(self, ind):
        for t in ["profile"]:###, "raw_data"]:
            table = self.find(QTableView, f"tableView_{t}")
            selectionModel = table.selectionModel()
            index = table.model().index(ind, 0)

            selectionModel.select(
                index,
                QItemSelectionModel.Rows |
                QItemSelectionModel.ClearAndSelect |
                QItemSelectionModel.Select
            )
            table.scrollTo(index)

    def update_table_selection_pol(self, ind):
        for t in ["pollutants"]:###, "raw_data"]:
            table = self.find(QTableView, f"tableView_{t}")
            selectionModel = table.selectionModel()
            index = table.model().index(ind, 0)

            selectionModel.select(
                index,
                QItemSelectionModel.Rows |
                QItemSelectionModel.ClearAndSelect |
                QItemSelectionModel.Select
            )
            table.scrollTo(index)

    def update(self, reach_id=None, profile_id=None, pol_id=None, timestamp=None):
        if reach_id is not None:
            self.plot_c.set_reach(reach_id)
            self.plot_m.set_reach(reach_id)
            self.plot_tot_c.set_reach(reach_id)
            self.plot_tot_eg.set_reach(reach_id)
            self.plot_tot_em.set_reach(reach_id)
            self.plot_tot_ed.set_reach(reach_id)

            self.update_table_selection_reach(reach_id)
            self.update_table_selection_profile(0)
            self.update_table_selection_pol(0)

        if profile_id is not None:
            self.plot_c.set_profile(profile_id)
            self.plot_m.set_profile(profile_id)
            self.plot_tot_c.set_profile(profile_id)
            self.plot_tot_eg.set_profile(profile_id)
            self.plot_tot_em.set_profile(profile_id)
            self.plot_tot_ed.set_profile(profile_id)

            self.update_table_selection_profile(profile_id)

        print("--**//++update pol_id before None: ", pol_id)

        if pol_id is not None:
            print("--**//++//**//** update pol_id: ", pol_id)
            self.plot_c.set_pollutant(pol_id)
            self.plot_m.set_pollutant(pol_id)

            self.update_table_selection_pol(pol_id)

        if timestamp is not None:
            self.plot_c.set_timestamp(timestamp)
            self.plot_m.set_timestamp(timestamp)
            self.plot_tot_c.set_timestamp(timestamp)
            self.plot_tot_eg.set_timestamp(timestamp)
            self.plot_tot_em.set_timestamp(timestamp)
            self.plot_tot_ed.set_timestamp(timestamp)

            self._table["raw_data"].timestamp = timestamp

        self.update_statusbar()

    def _set_current_reach(self):
        table = self.find(QTableView, f"tableView_reach")
        indexes = table.selectedIndexes()
        if len(indexes) == 0:
            return

        self.update(reach_id=indexes[0].row())

    def _set_current_profile(self):
        table = self.find(QTableView, f"tableView_profile")
        indexes = table.selectedIndexes()
        if len(indexes) == 0:
            return

        ind = indexes[0].row()
        self.update(profile_id=ind)
        ###self._slider_profile.setValue(ind)

    def _set_current_pol(self):
        table = self.find(QTableView, f"tableView_pollutants")
        indexes = table.selectedIndexes()
        if len(indexes) == 0:
            return

        ind = indexes[0].row()
        print("set pol id: ", ind)
        self.update(pol_id=ind)

    def _set_current_profile_raw_data(self):
        table = self.find(QTableView, f"tableView_raw_data")
        indexes = table.selectedIndexes()
        if len(indexes) == 0:
            return

        ind = indexes[0].row()
        self.update(profile_id=ind)
        self._slider_profile.setValue(ind)

    def _set_current_profile_slider(self):
        pid = self._slider_profile.value()
        self.update(profile_id=pid)

    def _set_current_timestamp(self):
        timestamp = self._timestamps[self._slider_time.value()]
        self.update(timestamp=timestamp)

    def _reload_plots(self):
        self.plot_c.results = self._results
        self.plot_m.results = self._results
        self.plot_tot_c.results = self._results
        self.plot_tot_eg.results = self._results
        self.plot_tot_em.results = self._results
        self.plot_tot_ed.results = self._results

        self.plot_c.draw()
        self.plot_m.draw()
        self.plot_tot_c.draw()
        self.plot_tot_eg.draw()
        self.plot_tot_em.draw()
        self.plot_tot_ed.draw()

    def _reload_slider(self):
        self._slider_time = self.find(QSlider, f"horizontalSlider_time")
        self._slider_time.setMaximum(len(self._timestamps) - 1)
        self._slider_time.setValue(len(self._timestamps) - 1)

    def _reload(self):
        logger.debug("Reload results...")
        self._results = self._results.reload()

        self._timestamps = sorted(list(self._results.get("timestamps")))

        self._reload_plots()
        ###self._reload_slider()

    def _copy(self):
        logger.info("TODO: copy")

    def _paste(self):
        logger.info("TODO: paste")

    def _undo(self):
        self._table.undo()

    def _redo(self):
        self._table.redo()

    # play / pause buttons
    def _update_slider(self):
        if self._slider_time.value() == self._slider_time.maximum():
            self._slider_time.setValue(self._slider_time.minimum())
        else:
            self._slider_time.setValue(self._slider_time.value()+1)

    def _next(self):
        self._slider_time.setValue(self._slider_time.value()+1)

    def _back(self):
        self._slider_time.setValue(self._slider_time.value()-1)

    def _first(self):
        self._slider_time.setValue(self._slider_time.minimum())

    def _last(self):
        self._slider_time.setValue(self._slider_time.maximum())

    def _pause(self):
        if self._button_play.isChecked():
            self._button_next.setEnabled(False)
            self._button_back.setEnabled(False)
            self._button_first.setEnabled(False)
            self._button_last.setEnabled(False)
            self._timer.start(100)
            self._button_play.setIcon(self._icon_pause)
        else:
            self._timer.stop()
            self._button_next.setEnabled(True)
            self._button_back.setEnabled(True)
            self._button_first.setEnabled(True)
            self._button_last.setEnabled(True)
            self._button_play.setIcon(self._icon_start)

    def export(self):
        self.file_dialog(
            select_file=False,
            callback=lambda d: self.export_to(d[0])
        )

    def export_to(self, directory):
        for reach in self._results.river.reachs:
            self.export_reach(reach, directory)

    def export_reach(self, reach, directory):
        name = reach.name
        name = name.replace(" ", "-")

        file_name = os.path.join(
            directory,
            f"reach_{name}.csv"
        )

        with open(file_name, 'w', newline='') as csvfile:
            writer = csv.writer(csvfile, delimiter=',',
                                quotechar='|', quoting=csv.QUOTE_MINIMAL)
            writer.writerow(["name", "kp", "data-file"])
            for profile in reach.profiles:
                p_file_name = os.path.join(
                    directory,
                    f"cs_{profile.geometry.id}.csv"
                )

                writer.writerow([
                    profile.name,
                    profile.kp,
                    p_file_name
                ])

                self.export_profile(reach, profile, p_file_name)

    def export_profile(self, reach, profile, file_name):
        with open(file_name, 'w', newline='') as csvfile:
            writer = csv.writer(csvfile, delimiter=',',
                                quotechar='|', quoting=csv.QUOTE_MINIMAL)

            writer.writerow(["timestamp", "z", "q"])
            timestamps = sorted(self._results.get("timestamps"))

            for ts in timestamps:
                writer.writerow([
                    ts,
                    profile.get_ts_key(ts, "Z"),
                    profile.get_ts_key(ts, "Q"),
                ])
